"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyfileBatch = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const fast_glob_1 = __importDefault(require("fast-glob"));
const promisify_1 = require("./promisify");
const copyfileAsync = (0, promisify_1.promisify)(fs_1.copyFile);
const mkdirAsync = (0, promisify_1.promisify)(fs_1.mkdir);
const accessAsync = (path) => {
    return new Promise(resolve => (0, fs_1.access)(path, err => resolve(err === null)));
};
/**
 * Get path and file name of files which was matched pattern
 * @param pattern
 * @returns
 */
function getFileInfosAsync(pattern) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield (0, fast_glob_1.default)(pattern);
        return files.map(file => {
            const ps = file.split("/");
            return {
                filepath: file,
                filename: ps[ps.length - 1],
            };
        });
    });
}
/**
 * Copy files to target directory
 * @param src   path from
 * @param dest  path to
 * @returns
 */
function copyFileEnhanceAsync(src, dest) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileInfos = yield getFileInfosAsync(src);
        // Create if directory or path is not existed
        const isExist = yield accessAsync((0, path_1.resolve)(process.cwd(), dest));
        if (!isExist) {
            yield mkdirAsync(dest, { recursive: true });
        }
        return fileInfos.map(({ filepath, filename }) => {
            const dist = `${dest.endsWith("/") ? dest : `${dest}/`}${filename}`;
            return copyfileAsync(filepath, dist);
        });
    });
}
/**
 * Copy files batch
 * @param {*} options
 */
function copyfileBatch(options) {
    const promises = options.reduce((defers, { src, dest }) => {
        const srcset = Array.isArray(src)
            ? src
            : [src];
        defers.push(...srcset.map(pattern => {
            return copyFileEnhanceAsync(pattern, (0, path_1.resolve)(process.cwd(), dest));
        }));
        return defers;
    }, []);
    return Promise.all(promises);
}
exports.copyfileBatch = copyfileBatch;
